const castContext = cast.framework.CastReceiverContext.getInstance();
const castDebugLogger = cast.debug.CastDebugLogger.getInstance();
const playerManager = castContext.getPlayerManager();

const TAG = "receiver.log";
const LIVE_MEDIA_URL = "https://test-streams.mux.dev/x36xhzz/x36xhzz.m3u8";
const LIVE_MEDIA_METADATA_SERVICE_URI =
  "https://us-central1-cast-tses.cloudfunctions.net/io-2019-live";

/**
 * Start playing the live stream as soon as the receiver is finished loading.
 */
castContext.addEventListener(cast.framework.system.EventType.READY, (event) => {
  const mediaInfo = new cast.framework.messages.MediaInformation();
  const loadRequest = new cast.framework.messages.LoadRequestData(mediaInfo);

  castDebugLogger.info(TAG, "Receiver started, initiating load request.");
  console.log(TAG, "Receiver started, initiating load request.");
  castDebugLogger.info(
    "ServiceWorker",
    "serviceWorker" in navigator ? "available" : "not available"
  );
  playerManager.load(loadRequest);
});

/**
 * Load interceptor
 */
playerManager.setMessageInterceptor(
  cast.framework.messages.MessageType.LOAD,
  (request) => {
    var adapter = {
      getMeta: function (id) {
        return {
          pl: "testPlayer1008",
          plv: "1.0.0",
          sx: window.innerWidth,
          sy: window.innerHeight,
        };
      },
      getPosition: function (id) {
        return Math.round(playerManager.getCurrentTimeSec());
      },
      getDuration: function (id) {
        castDebugLogger.info(
          "Duration",
          Math.round(playerManager.getDurationSec())
        );
        console.log("duration: ", Math.round(playerManager.getDurationSec()));
        return Math.round(playerManager.getDurationSec());
      },
    };

    window.onbeforeunload = function (e) {
      sensors.unload();
    };
    var desc = {
      sx: window.innerWidth,
      sy: window.innerHeight,
      stream: "stream-test",
    };
    castDebugLogger.info(
      "media information",
      playerManager.getMediaInformation()
    );
    castDebugLogger.info("desc", desc);

    /**
     * -----------------------------------------------
     * WARNING! Use code below if you use BARB sensor
     * -----------------------------------------------
     */
    /*
    var nextUid = sensors.getNextUID();
    handle = sensors.trackWithUID("testplayer", desc, adapter, nextUid);
    */

    /**
     * -----------------------------------------------
     * WARNING! Use code below if you use OTHER THAN BARB sensor
     * -----------------------------------------------
     */

    handle = sensors.track("testplayer", desc, adapter);

    castDebugLogger.info("TRACKING", handle);
    sensors.debug = function (v) {
      castDebugLogger.info("Request", v);
      console.log("Request", v);
    };
    castDebugLogger.info("request", request);
    console.log("Request", request);
    castDebugLogger.info(TAG, "Intercepting LOAD request");
    console.log(TAG, "Intercepting LOAD request");
    request.autoplay = true;
    request.media.streamType = cast.framework.messages.StreamType.LIVE;
    request.media.contentUrl = LIVE_MEDIA_URL;

    loadGuideData();

    playerManager.removeSupportedMediaCommands(
      cast.framework.messages.Command.SEEK,
      true
    );
    castDebugLogger.info("SDK", sensors);
    console.log("SDK", sensors);

    return request;
  }
);

/**
 * Seek a seek interceptor is called whenever CAF receives a request to seek to
 * a different location in media. This interceptor can be used to modify that
 * seek request or disable seeking completely.
 * @param {cast.framework.messages.MessageType.SEEK} The message to intercept.
 */
playerManager.setMessageInterceptor(
  cast.framework.messages.MessageType.SEEK,
  (seekData) => {
    // if the SEEK supported media command is disabled, block seeking
    if (
      !(
        playerManager.getSupportedMediaCommands() &
        cast.framework.messages.Command.SEEK
      )
    ) {
      castDebugLogger.info(TAG, "Seek blocked.");
      return null;
    }

    return seekData;
  }
);

/**
 * Gets the current program guide data from our Google Cloud Function
 * @return {cast.framework.messages.MediaMetadata[]} Latest program guide data
 */
function loadGuideData() {
  return fetch(LIVE_MEDIA_METADATA_SERVICE_URI)
    .then((response) => response.json())
    .then(function (data) {
      castDebugLogger.info("meta", data);
      const containerMetadata = new cast.framework.messages.ContainerMetadata();
      containerMetadata.sections = data;
      castDebugLogger.info("containerMetadata", containerMetadata);
      console.log("containerMetadata", containerMetadata);

      playerManager.getQueueManager().setContainerMetadata(containerMetadata);
    });
}
